@@ -10,9 +10,6 @@ static gboolean tasklist_icon_check_mini (TasklistTask *task);
 static gboolean tasklist_icon_check_x (TasklistTask *task);
 static void tasklist_icon_set_minimized (TasklistTask *task);
 
-extern GtkWidget *area; /* The drawing area used to display tasks */
-extern TasklistIcon *unknown_icon; /* The unknown icon */
-
 #define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
 
 /* Shamelessly stolen from gwmh.c by Tim Janik */
@@ -22,6 +19,9 @@ tasklist_icon_get_pixmap (TasklistTask *task)
 {
 	XWMHints *wmhints;
 	Pixmap pixmap;
+
+	if (task == NULL)
+		return 0;
 	
 	wmhints = XGetWMHints (GDK_DISPLAY (), task->gwmh_task->xwin);
 
@@ -133,7 +133,8 @@ static gboolean
 tasklist_icon_check_mini (TasklistTask *task)
 {
 	GdkGC *gc;
-	int x, y, b, width, height, depth;
+	int x, y;
+	guint b, width, height, depth;
 	guint32 *atomdata;
 	Window root;
 	GdkImage *image;
@@ -160,13 +161,17 @@ tasklist_icon_check_mini (TasklistTask *task)
 	if (!atomdata)
 		return FALSE;
 	
-	if (!atomdata[0])
+	if (!atomdata[0]) {
+		g_free (atomdata);
 		return FALSE;
+	}
+
+	gdk_error_trap_push ();
 
 	/* Get icon size and depth */
 	XGetGeometry (xdisplay, (Drawable)atomdata[0], &root, &x, &y,
 		      &width, &height, &b, &depth);
-	
+
 	/* Create a new GdkPixmap and copy the mini icon pixmap to it */
 	pixmap = gdk_pixmap_new (NULL, width, height, depth);
 	gc = gdk_gc_new (pixmap);
@@ -176,17 +181,17 @@ tasklist_icon_check_mini (TasklistTask *task)
 	
 	pixbuf = gdk_pixbuf_get_from_drawable (NULL,
 					       pixmap,
-					       gtk_widget_get_colormap (area),
+					       gtk_widget_get_colormap (task->tasklist->area),
 					       0, 0,
 					       0, 0,
 					       width, height);
 	gdk_pixmap_unref (pixmap);
 	
-	if (atomdata[1]) {
+	if (size > 1 && atomdata[1]) {
 		mask = gdk_pixmap_new (NULL, width, height, depth);
 		gc = gdk_gc_new (mask);
-		gdk_gc_set_background (gc, &area->style->black);
-		gdk_gc_set_foreground (gc, &area->style->white);
+		gdk_gc_set_background (gc, &task->tasklist->area->style->black);
+		gdk_gc_set_foreground (gc, &task->tasklist->area->style->white);
 		XCopyPlane (GDK_DISPLAY (), atomdata[1], GDK_WINDOW_XWINDOW (mask),
 			    GDK_GC_XGC (gc), 0, 0, width, height, 0, 0, 1);
 		gdk_gc_unref (gc);
@@ -206,6 +211,10 @@ tasklist_icon_check_mini (TasklistTask *task)
 		
 		gdk_pixmap_unref (mask);
 	}
+
+	gdk_error_trap_pop ();
+
+	g_free (atomdata);
 	
 	task->icon->normal = pixbuf;
 	
@@ -226,32 +235,41 @@ tasklist_icon_check_x (TasklistTask *task)
 	unsigned int border_width;
 	unsigned int depth;
 	guchar *data;
+
+	gdk_error_trap_push ();
 	
 	wmhints = XGetWMHints (GDK_DISPLAY (), task->gwmh_task->xwin);
 
-	if (!wmhints)
-		return FALSE;
-	
-	if (!(wmhints->flags & IconPixmapHint)) {
-		XFree (wmhints);
+	if (!wmhints) {
+		gdk_flush ();
+		gdk_error_trap_pop ();
 		return FALSE;
 	}
 	
+	if (!(wmhints->flags & IconPixmapHint)) {
+		XFree (wmhints);
+		gdk_flush ();
+		gdk_error_trap_pop ();
+		return FALSE;
+	}
+
 	XGetGeometry (GDK_DISPLAY (), wmhints->icon_pixmap, &root,
 		      &x, &y, &width, &height,
 		      &border_width, &depth);
 	
 	if (width > 65535 || height > 65535) {
 		XFree (wmhints);
+		gdk_flush ();
+		gdk_error_trap_pop ();
 		return FALSE;
 	}
 	
-	pixmap = gdk_pixmap_new (area->window, width, height, -1);	
+	pixmap = gdk_pixmap_new (task->tasklist->area->window, width, height, -1);	
 	gc = gdk_gc_new (pixmap);
 
 	if (depth == 1) {
-		gdk_gc_set_background (gc, &area->style->white);
-		gdk_gc_set_foreground (gc, &area->style->black);
+		gdk_gc_set_background (gc, &task->tasklist->area->style->white);
+		gdk_gc_set_foreground (gc, &task->tasklist->area->style->black);
 		XCopyPlane (GDK_DISPLAY (), wmhints->icon_pixmap, GDK_WINDOW_XWINDOW (pixmap),
 			   GDK_GC_XGC (gc), 0, 0, width, height, 0, 0, 1);
 	}
@@ -259,7 +277,7 @@ tasklist_icon_check_x (TasklistTask *task)
 		XCopyArea (GDK_DISPLAY (), wmhints->icon_pixmap, GDK_WINDOW_XWINDOW (pixmap),
 			   GDK_GC_XGC (gc), 0, 0, width, height, 0, 0);
 	}
-	pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, gtk_widget_get_colormap (area), 
+	pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, gtk_widget_get_colormap (task->tasklist->area), 
 					       0, 0,
 					       0, 0, width, height);
 	gdk_gc_destroy (gc);
@@ -271,8 +289,8 @@ tasklist_icon_check_x (TasklistTask *task)
 	if (wmhints->flags & IconMaskHint) {
 	       mask = gdk_pixmap_new (NULL, width, height, depth);
 	       gc = gdk_gc_new (mask);
-	       gdk_gc_set_background (gc, &area->style->black);
-	       gdk_gc_set_foreground (gc, &area->style->white);
+	       gdk_gc_set_background (gc, &task->tasklist->area->style->black);
+	       gdk_gc_set_foreground (gc, &task->tasklist->area->style->white);
 	       XCopyPlane (GDK_DISPLAY (), wmhints->icon_mask, GDK_WINDOW_XWINDOW (mask),
 			   GDK_GC_XGC (gc), 0, 0, width, height, 0, 0, 1);
 	       gdk_gc_unref (gc);
@@ -306,6 +324,9 @@ tasklist_icon_check_x (TasklistTask *task)
 
 	XFree (wmhints);
 
+	gdk_flush ();
+	gdk_error_trap_pop ();
+
 	return TRUE;
 
 }
@@ -313,10 +334,13 @@ tasklist_icon_check_x (TasklistTask *task)
 void
 tasklist_icon_set (TasklistTask *task)
 {
-	task->icon = g_new (TasklistIcon, 1);
+	if (task == NULL)
+		return;
+	
+	task->icon = g_new0 (TasklistIcon, 1);
 
-	task->icon->normal = unknown_icon->normal;
-	task->icon->minimized = unknown_icon->minimized;
+	task->icon->normal    = task->tasklist->unknown_icon->normal;
+	task->icon->minimized = task->tasklist->unknown_icon->minimized;
 
 	if (!tasklist_icon_check_x (task))
 		tasklist_icon_check_mini (task);
@@ -329,19 +353,22 @@ static void
 tasklist_icon_set_minimized (TasklistTask *task)
 {
 	if (task->icon->minimized &&
-		task->icon->minimized != unknown_icon->minimized)
+	    task->icon->minimized != task->tasklist->unknown_icon->minimized)
 		gdk_pixbuf_unref (task->icon->minimized);
 
-	task->icon->minimized = tasklist_icon_create_minimized_icon (task->icon->normal);
+	task->icon->minimized = tasklist_icon_create_minimized_icon (task->tasklist, task->icon->normal);
 }
 
 void
 tasklist_icon_destroy (TasklistTask *task)
 {
-	if (task->icon->normal != unknown_icon->normal)
+	if (task == NULL)
+		return;
+	
+	if (task->icon->normal != task->tasklist->unknown_icon->normal)
 		gdk_pixbuf_unref (task->icon->normal);
 	
-	if (task->icon->minimized != unknown_icon->minimized)
+	if (task->icon->minimized != task->tasklist->unknown_icon->minimized)
 		gdk_pixbuf_unref (task->icon->minimized);
 
 	g_free (task->icon);
@@ -349,7 +376,7 @@ tasklist_icon_destroy (TasklistTask *task)
 
 /* Stolen from gnome-pixmap.c */
 GdkPixbuf *
-tasklist_icon_create_minimized_icon (GdkPixbuf *pixbuf)
+tasklist_icon_create_minimized_icon (Tasklist *tasklist, GdkPixbuf *pixbuf)
 {
 	GdkPixbuf *target;
 	gint i, j;
@@ -360,7 +387,7 @@ tasklist_icon_create_minimized_icon (GdkPixbuf *pixbuf)
 	gint32 red, green, blue;
 	GdkColor color;
 	
-	color = area->style->bg[GTK_STATE_NORMAL];
+	color = tasklist->area->style->bg[GTK_STATE_NORMAL];
 	red = color.red / 255;
 	blue = color.blue / 255;
 	green = color.green / 255;
