@@ -24,6 +24,10 @@
 extern gboolean panel_applet_in_drag;
 extern GSList *panel_list;
 
+extern int applets_to_sync;
+extern int panels_to_sync;
+extern int need_complete_save;
+
 /*global settings*/
 extern int pw_explicit_step;
 extern int pw_drawer_step;
@@ -386,13 +390,13 @@ basep_pos_get_hide_size (BasePWidget *basep,
 	case ORIENT_DOWN:
 		*h = (basep->state == BASEP_AUTO_HIDDEN)
 			? pw_minimized_size
-			: basep->hidebutton_n->allocation.height;
+			: get_requisition_height (basep->hidebutton_n);
 		break;
 	case ORIENT_RIGHT:
 	case ORIENT_LEFT:
 		*w = (basep->state == BASEP_AUTO_HIDDEN)
 			? pw_minimized_size
-			: basep->hidebutton_e->allocation.width;
+			: get_requisition_width (basep->hidebutton_e);
 		break;
 	}
 	*w = MAX (*w, 1);
@@ -411,11 +415,13 @@ basep_pos_get_hide_pos (BasePWidget *basep,
 		break;
 	case ORIENT_RIGHT:
 		*x += w - ((basep->state == BASEP_AUTO_HIDDEN)
-			    ? pw_minimized_size : basep->hidebutton_w->allocation.width);
+			   ? pw_minimized_size
+			   : get_requisition_width (basep->hidebutton_w));
 		break;
 	case ORIENT_DOWN:
 		*y += h - ((basep->state == BASEP_AUTO_HIDDEN)
-			   ? pw_minimized_size : basep->hidebutton_s->allocation.height);
+			   ? pw_minimized_size
+			   : get_requisition_width (basep->hidebutton_s));
 		break;
 	}
 }
@@ -1069,6 +1075,17 @@ basep_widget_update_winhints (BasePWidget *basep)
 		} else {
 			layer = WIN_LAYER_DOCK;
 		}
+
+		/* drawers are always in DOCK, or NORMAL */
+		if ( IS_DRAWER_WIDGET(w) &&
+		     ! global_config.normal_layer)
+			layer = WIN_LAYER_DOCK;
+
+		/* if in autohiding mode, then we always want dock by default
+		 */
+		if (basep->mode == BASEP_AUTO_HIDE)
+			layer = WIN_LAYER_DOCK;
+
 	} else if (basep->level == BASEP_LEVEL_BELOW) {
 		layer = WIN_LAYER_BELOW;
 	} else if (basep->level == BASEP_LEVEL_NORMAL) {
@@ -1080,11 +1097,6 @@ basep_widget_update_winhints (BasePWidget *basep)
 	switch (basep->state) {
 	case BASEP_SHOWN:
 	case BASEP_MOVING:
-		/* drawers are always in DOCK, or NORMAL */
-		if( IS_DRAWER_WIDGET(w) &&
-		    basep->level == BASEP_LEVEL_DEFAULT &&
-		    ! global_config.normal_layer)
-			layer = WIN_LAYER_DOCK;
 
 		gnome_win_hints_set_layer (w, layer);
 		gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS |
@@ -1392,6 +1404,7 @@ basep_widget_change_params (BasePWidget *basep,
 		gtk_signal_emit(GTK_OBJECT(basep),
 				basep_widget_signals[STATE_CHANGE_SIGNAL],
 				state);
+		panels_to_sync = TRUE;
 	}
 
 	panel_widget_change_params(PANEL_WIDGET(basep->panel),
@@ -1522,6 +1535,7 @@ basep_widget_explicit_hide (BasePWidget *basep, BasePState state)
 	gtk_signal_emit(GTK_OBJECT(basep),
 			basep_widget_signals[STATE_CHANGE_SIGNAL],
 			state);
+	panels_to_sync = TRUE;
 
 	/* if the app did any updating of the interface, flush that for us*/
 	gdk_flush();
@@ -1598,6 +1612,7 @@ basep_widget_explicit_show (BasePWidget *basep)
 	gtk_signal_emit(GTK_OBJECT(basep),
 			basep_widget_signals[STATE_CHANGE_SIGNAL],
 			BASEP_SHOWN);
+	panels_to_sync = TRUE;
 }
 
 void
@@ -1940,6 +1955,7 @@ basep_widget_set_state (BasePWidget *basep, BasePState state,
 		gtk_signal_emit(GTK_OBJECT(basep),
 				basep_widget_signals[STATE_CHANGE_SIGNAL],
 				state);
+	panels_to_sync = TRUE;
 }
 
 /*****
