@@ -185,7 +185,7 @@ static void
 about_cb (GtkWidget *w, gpointer data)
 {
 	char *v[2] = { "gnome-about" };
-	gnome_execute_async (NULL, 1, v);
+	gnome_execute_async (g_get_home_dir (), 1, v);
 }
 #endif
 
@@ -194,7 +194,7 @@ gmc_client (GtkWidget *w, gpointer data)
 {
 	char *v[3] = { "gmc-client" };
 	v[1] = data;
-	if(gnome_execute_async (NULL, 2, v) < 0)
+	if(gnome_execute_async (g_get_home_dir (), 2, v) < 0)
 		panel_error_dialog(_("Cannot execute the gmc-client program,\n"
 				     "perhaps gmc is not installed"));
 }
@@ -204,7 +204,7 @@ gnomecal_client (GtkWidget *w, gpointer data)
 {
 	char *v[4] = { "gnomecal", "--view" };
 	v[2] = data;
-	if(gnome_execute_async (NULL, 3, v) < 0)
+	if(gnome_execute_async (g_get_home_dir (), 3, v) < 0)
 		panel_error_dialog(_("Cannot execute the gnome calendar,\n"
 				     "perhaps it's not installed.\n"
 				     "It is in the gnome-pim package."));
@@ -253,13 +253,62 @@ append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 }
 #endif
 
-static void
+static GtkWidget *
 append_gmc_item (GtkWidget *menu, const char *label, char *flag)
 {
-	GtkWidget *item = gtk_menu_item_new_with_label (label);
+	GtkWidget *item;
+
+	item = gtk_menu_item_new_with_label (label);
 	gtk_menu_append (GTK_MENU (menu), item);
 	gtk_signal_connect (GTK_OBJECT (item), "activate",
 			    GTK_SIGNAL_FUNC (gmc_client), flag);
+
+	return item;
+}
+
+static gboolean
+display_gmc_menu (void)
+{
+	static gboolean checked_path = FALSE;
+	static gboolean got_gmc = FALSE;
+
+	if ( ! checked_path) {
+		char *gmc_client;
+		gmc_client = panel_is_program_in_path ("gmc-client");
+
+		if (gmc_client == NULL)
+			got_gmc = FALSE;
+		else
+			got_gmc = TRUE;
+
+		g_free (gmc_client);
+		checked_path = TRUE;
+	}
+
+	if ( ! got_gmc)
+		return FALSE;
+
+	if (xstuff_nautilus_desktop_present ())
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+desktop_selected (GtkWidget *widget, gpointer data)
+{
+	GList *gmc_menu_items = data;
+	GList *li;
+	gboolean gmc_menu = display_gmc_menu ();
+
+	for (li = gmc_menu_items; li != NULL; li = li->next) {
+		GtkWidget *item = li->data;
+
+		if (gmc_menu)
+			gtk_widget_show (item);
+		else
+			gtk_widget_hide (item);
+	}
 }
 
 static void
@@ -277,6 +326,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 		N_("By Time Last Changed"),  "--arrange-desktop-icons=ctime",
 		NULL
 	};
+	GList *gmc_menu_items = NULL;
 
 	menu = scroll_menu_new ();
 
@@ -284,6 +334,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 		append_gmc_item (menu, _(arrange[i]), arrange[i+1]);
 
 	item = gtk_menu_item_new_with_label (_("Arrange Icons"));
+	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 
 	add_tearoff (GTK_MENU (menu));
@@ -291,12 +342,19 @@ append_desktop_menu (GtkWidget *menu_bar)
 	menu = scroll_menu_new ();
 
 	gtk_menu_append (GTK_MENU (menu), item);
-	add_menu_separator (menu);
 
-	append_gmc_item (menu, _("Rescan Desktop Directory"), "--rescan-desktop");
-	append_gmc_item (menu, _("Rescan Desktop Devices"), "--rescan-desktop-devices");
+	item = add_menu_separator (menu);
+	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
 
-	add_menu_separator (menu);
+	item = append_gmc_item (menu, _("Rescan Desktop Directory"),
+				"--rescan-desktop");
+	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
+	item = append_gmc_item (menu, _("Rescan Desktop Devices"),
+				"--rescan-desktop-devices");
+	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
+
+	item = add_menu_separator (menu);
+	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
 	
 	char_tmp = panel_is_program_in_path ("xscreensaver");
 	if (char_tmp) {	
@@ -321,6 +379,14 @@ append_desktop_menu (GtkWidget *menu_bar)
 	item = gtk_menu_item_new_with_label (_(" Desktop "));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+
+	gtk_signal_connect_full (GTK_OBJECT (menu), "show",
+				 GTK_SIGNAL_FUNC (desktop_selected),
+				 NULL,
+				 gmc_menu_items,
+				 (GtkDestroyNotify) g_list_free,
+				 FALSE,
+				 FALSE);
 }
 
 static GtkWidget *
