@@ -19,10 +19,12 @@
 
 #include "xstuff.h"
 
-GdkAtom KWM_MODULE = 0;
-GdkAtom KWM_MODULE_DOCKWIN_ADD = 0;
-GdkAtom KWM_MODULE_DOCKWIN_REMOVE = 0;
-GdkAtom KWM_DOCKWINDOW = 0;
+static GdkAtom KWM_MODULE = 0;
+static GdkAtom KWM_MODULE_DOCKWIN_ADD = 0;
+static GdkAtom KWM_MODULE_DOCKWIN_REMOVE = 0;
+static GdkAtom KWM_DOCKWINDOW = 0;
+static GdkAtom NAUTILUS_DESKTOP_WINDOW_ID = 0;
+static GdkAtom _NET_WM_DESKTOP = 0;
 
 extern GList *check_swallows;
 
@@ -66,25 +68,23 @@ try_adding_status(guint32 winid)
 }
 
 static void
-try_checking_swallows (guint32 winid)
+try_checking_swallows (GwmhTask *task)
 {
-	char *tit = NULL;
-	if (XFetchName (GDK_DISPLAY (), winid, &tit) &&
-	    tit != NULL) {
-		GList *li;
-		for (li = check_swallows; li;
-		     li = li->next) {
-			Swallow *swallow = li->data;
-			if (strstr (tit, swallow->title) != NULL) {
-				swallow->wid = winid;
-				gtk_socket_steal (GTK_SOCKET (swallow->socket),
-						  swallow->wid);
-				check_swallows = 
-					g_list_remove (check_swallows, swallow);
-				break;
-			}
+	GList *li;
+
+	if (!task->name)
+		return;
+
+	for (li = check_swallows; li; li = li->next) {	     
+		Swallow *swallow = li->data;
+		if (strstr (task->name, swallow->title) != NULL) {
+			swallow->wid = task->xwin;
+			gtk_socket_steal (GTK_SOCKET (swallow->socket),
+					  swallow->wid);
+			check_swallows = 
+				g_list_remove (check_swallows, swallow);
+			break;
 		}
-		XFree (tit);
 	}
 }
 
@@ -98,7 +98,7 @@ xstuff_go_through_client_list (void)
 	for (li = gwmh_task_list_get (); li != NULL; li = li->next) {
 		GwmhTask *task = li->data;
 		if (check_swallows != NULL)
-			try_checking_swallows (task->xwin);
+			try_checking_swallows (task);
 		try_adding_status (task->xwin);
 	}
 	gdk_flush();
@@ -142,7 +142,7 @@ task_notifier (gpointer func_data,
 	case GWMH_NOTIFY_INFO_CHANGED:
 	case GWMH_NOTIFY_NEW:
 		if (check_swallows != NULL)
-			try_checking_swallows (task->xwin);
+			try_checking_swallows (task);
 		try_adding_status (task->xwin);
 		break;
 	default:
@@ -161,6 +161,10 @@ xstuff_init (void)
 	KWM_MODULE_DOCKWIN_REMOVE =
 		gdk_atom_intern ("KWM_MODULE_DOCKWIN_REMOVE", FALSE);
 	KWM_DOCKWINDOW = gdk_atom_intern ("KWM_DOCKWINDOW", FALSE);
+	NAUTILUS_DESKTOP_WINDOW_ID =
+		gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE);
+	_NET_WM_DESKTOP =
+		gdk_atom_intern ("_NET_WM_DESKTOP", FALSE);
 
 	gwmh_init ();
 
@@ -175,6 +179,39 @@ xstuff_init (void)
 	xstuff_go_through_client_list ();
 }
 
+gboolean
+xstuff_nautilus_desktop_present (void)
+{
+	gboolean ret = FALSE;
+	guint32 *data;
+	int size;
+
+	gdk_error_trap_push ();
+	data = get_typed_property_data (GDK_DISPLAY (),
+					GDK_ROOT_WINDOW (),
+					NAUTILUS_DESKTOP_WINDOW_ID,
+					XA_WINDOW,
+					&size, 32);
+	if (data != NULL &&
+	    *data != 0) {
+		guint32 *desktop;
+		desktop = get_typed_property_data (GDK_DISPLAY (),
+						   *data,
+						   _NET_WM_DESKTOP,
+						   XA_CARDINAL,
+						   &size, 32);
+		if (size > 0)
+			ret = TRUE;
+		g_free (desktop);
+	}
+	g_free (data);
+
+	gdk_flush ();
+	gdk_error_trap_pop ();
+
+	return ret;
+}
+
 void
 xstuff_set_simple_hint (GdkWindow *w, GdkAtom atom, int val)
 {
