@@ -6,11 +6,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-extern GdkAtom KWM_MODULE;
-extern GdkAtom KWM_MODULE_DOCKWIN_ADD;
-extern GdkAtom KWM_MODULE_DOCKWIN_REMOVE;
-extern GdkAtom KWM_DOCKWINDOW;
-
 void xstuff_init (void);
 void xstuff_set_simple_hint (GdkWindow *w, GdkAtom atom, int val);
 void xstuff_setup_kde_dock_thingie (GdkWindow *w);
@@ -33,4 +28,6 @@ void xstuff_go_through_client_list (void);
 
 void xstuff_set_no_group_and_no_input (GdkWindow *win);
 
+gboolean xstuff_nautilus_desktop_present (void);
+
 #endif
